@@ -4,11 +4,11 @@
 
  Simulation of a random field by sequential method
 
- Copyright (C) 2001 -- 2011 Martin Schlather, 
+ Copyright (C) 2001 -- 2013 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
+as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -26,42 +26,68 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdlib.h>
  
 #include "RF.h"
+#include "randomshape.h"
 #include <R_ext/Lapack.h>
 //#include <R_ext/Linpack.h>
 
+#define SEQU_MAX (COMMON_GAUSS + 1)
+#define SEQU_BACK (COMMON_GAUSS + 2)
+#define SEQU_INIT (COMMON_GAUSS + 3)
 
 
 bool debugging = true;
 
-void sequential_destruct(void ** S)
-{
-  if (*S!=NULL) {
-    sequential_storage *x;
-    x = *((sequential_storage**)S);    
-    if (x->U11!=NULL) free(x->U11); 
-    if (x->U22!=NULL) free(x->U22); 
-    if (x->MuT!=NULL) free(x->MuT); 
-    if (x->G!=NULL) free(x->G);
-    if (x->Cov21!=NULL) free(x->Cov21);
-    if (x->Inv22!=NULL) free(x->Inv22);
-    if (x->res0!=NULL) free(x->res0);
-    free(*S);
-    *S = NULL;
-  }
+
+int check_sequential(cov_model *cov) {
+#define nsel 4
+  cov_model *next=cov->sub[0];
+  int err,
+    dim = cov->tsdim; // taken[MAX DIM],
+  sequ_param *gp  = &(GLOBAL.sequ);
+
+  ROLE_ASSERT(ROLE_GAUSS);
+
+  if ((err = check_common_gauss(cov)) != NOERROR) return err;
+  kdefault(cov, SEQU_MAX, gp->max);
+  kdefault(cov, SEQU_BACK, gp->back);
+  kdefault(cov, SEQU_INIT, gp->initial);
+  if ((err = checkkappas(cov)) != NOERROR) return err;
+ 
+  if (cov->tsdim != cov->xdimprev || cov->tsdim != cov->xdimown) 
+    return ERRORDIM;
+
+  if ((err = CHECK(next, dim, dim, PosDefType, XONLY, SYMMETRIC,
+		     SUBMODEL_DEP, ROLE_COV)) != NOERROR) return err;
+  if (next->pref[Sequential] == PREF_NONE) return ERRORPREFNONE;
+  setbackward(cov, next);
+
+  return NOERROR;
 }
 
-void SetParamSequential(int *action, int *maxvariables, int *back, int *initial)
-{
-  sequ_param  *gp = &(GLOBAL.sequ);
-  if (*action) {
-    gp->max  = *maxvariables;
-    gp->back = *back < 1 ? 1 : *back;
-    gp->initial= *initial;
-  } else {
-    *maxvariables  = gp->max;
-    *back  = gp->back;
-    *initial = gp->initial; 
-   }
+
+void range_sequential(cov_model *cov, range_type *range) {
+  range_common_gauss(cov, range);
+
+  range->min[SEQU_MAX] = 0;
+  range->max[SEQU_MAX] = RF_INF;
+  range->pmin[SEQU_MAX] = 100;
+  range->pmax[SEQU_MAX] = 8000;
+  range->openmin[SEQU_MAX] = false;
+  range->openmax[SEQU_MAX] = true; 
+
+  range->min[SEQU_BACK] = 0;
+  range->max[SEQU_BACK] = RF_INF;
+  range->pmin[SEQU_BACK] = 0.1;
+  range->pmax[SEQU_BACK] = 10;
+  range->openmin[SEQU_BACK] = true;
+  range->openmax[SEQU_BACK] = true; 
+
+  range->min[SEQU_INIT] = RF_NEGINF;
+  range->max[SEQU_INIT] = RF_INF;
+  range->pmin[SEQU_INIT] = -10;
+  range->pmax[SEQU_INIT] = 10;
+  range->openmin[SEQU_INIT] = false;
+  range->openmax[SEQU_INIT] = true; 
 }
 
 // (U1, U2) ~ N(0, S) =>
@@ -69,148 +95,120 @@ void SetParamSequential(int *action, int *maxvariables, int *back, int *initial)
 
 // start mit S_22; dann nur zeilenweise + Einschwingen
 
-int init_sequential(method_type *meth){
-  cov_model *cov = meth->cov;
-  cov_fct *C=CovList + cov->nr;
-  covfct cf = C->cov;
-  location_type *loc = meth->loc;
-  globalparam *gp = meth->gp;
-  sequ_param* lp = &(gp->sequ);
-  long Err=NOERROR, totpnts=0, totpntsSQ, spatialpntsSQ=0, i,
-      spatialpntsSQback;
+int init_sequential(cov_model *cov, storage VARIABLE_IS_NOT_USED *s){
+  cov_model *next = cov->sub[0];
+  // cov_fct *C=CovList + next->gatternr;
+  // covfct cf = C->cov;
+  location_type *loc = Loc(cov);
   double 
-    *xx = NULL,
+    //*caniso = loc->caniso,
+     *xx = NULL,
     *G = NULL, 
     *U11 = NULL, 
     *COV21 = NULL,
     *MuT = NULL, 
     *U22 = NULL, 
-      *Inv22 = NULL;
+    *Inv22 = NULL;
   res_type
     *res0 = NULL;
   int withoutlast, d, endfor, l, store,
     dim=cov->tsdim,
     spatialdim = dim - 1,
-    spatialpnts=0,
-    back=0, 
-    initial=0,
-    PL = gp->general.printlevel;
+    spatialpnts = loc->totalpoints / loc->length[spatialdim],
+    max = ((int *) cov->p[SEQU_MAX])[0],
+    back= ((int *) cov->p[SEQU_BACK])[0], 
+    initial= ((int *) cov->p[SEQU_INIT])[0];
   sequential_storage* S = NULL;
-  bool storing = gp->general.storing;
+  long totpntsSQ, i, spatialpntsSQback,
+    err=NOERROR,
+    totpnts = back * spatialpnts, 
+    spatialpntsSQ=spatialpnts * spatialpnts;
+  bool 
+    storing = GLOBAL.warn.stored_init,
+    Time = loc->Time;
  
-  assert(meth->S == NULL); // ist in simu.cc sichergestellt, schadet aber nicht
-  SET_DESTRUCT(sequential_destruct);
-  bool Time = loc->Time;
-
-  if (meth->caniso != NULL || meth->cproj!=NULL || meth->cscale != 1.0) {
-      Err = ERRORPREVDOLLAR;
-      goto ErrorHandling;
+  if (cov->role == ROLE_COV) {
+    return NOERROR;
   }
-   
-  if (!loc->grid && !Time) {
-    Err = ERRORLASTGRID; 
-    goto ErrorHandling;
-  }  
-  if (C->implemented[Sequential] != IMPLEMENTED) {
-    Err=ERRORNOTDEFINED; 
+  
+  ROLE_ASSERT_GAUSS;
+  cov->method = Sequential;
+ 
+  if (loc->distances) return ERRORFAILED;
+  if (back < 1) back = max / spatialpnts;
+    
+  if (!loc->grid && !Time) 
+    GERR("last component must be truely given by a non-trivial grid");
+
+  if (CovList[next->nr].implemented[Sequential] != IMPLEMENTED) {
+    err=ERRORNOTDEFINED; 
     goto ErrorHandling;
   }
   
   if (cov->vdim > 1) {
-      Err=ERRORNOMULTIVARIATE; 
+      err=ERRORNOMULTIVARIATE; 
       goto ErrorHandling;   
   }
 
-  spatialpnts = loc->totalpoints / loc->length[spatialdim];
-
-  spatialpntsSQ = spatialpnts * spatialpnts;
-  totpnts = (lp->back < 1 ? 1 : lp->back) * spatialpnts;
-  if (totpnts > lp->max) {
-      sprintf(ERRORSTRING_OK, 
-	    "number of points less than RFparameters()$sequential.max (%d)",
-	      (int) lp->max);
-       sprintf(ERRORSTRING_WRONG,"%d * %d = %ld", 
-	       (lp->back < 1 ? 1 : lp->back), 
-	       spatialpnts, totpnts);
-       Err=ERRORCOVFAILED; goto ErrorHandling;
+  if (totpnts > max) {
+    sprintf(ERRORSTRING_OK, "number of points larger than 'max' (%d)", max);
+    sprintf(ERRORSTRING_WRONG,"%d * %d = %ld", back, spatialpnts, totpnts);
+    err=ERRORCOVFAILED; goto ErrorHandling;
   }
 
-  Err=ERRORMEMORYALLOCATION;  
-  if ((meth->S = malloc(sizeof(sequential_storage))) == NULL)
-    goto ErrorHandling;
-  S = (sequential_storage*) meth->S;
-
-  back = lp->back;
-  if (back < 1) {
-    back = lp->max / spatialpnts;
-  }
-  totpnts = back * spatialpnts;
+ 
   totpntsSQ = totpnts * totpnts;
   spatialpntsSQback = totpnts * spatialpnts;
 
-  initial = lp->initial;
   if (initial < 0) initial = back - initial;
-
-  if ((U22 =(double *) malloc(sizeof(double) * totpntsSQ))==NULL)
-      goto ErrorHandling;
-  if ((Inv22 =(double *) malloc(sizeof(double) * totpntsSQ))==NULL)
-      goto ErrorHandling;
-  if ((COV21 =(double *) malloc(sizeof(double) * spatialpntsSQback))==NULL)
+   
+  if (cov->Sseq != NULL) free(cov->Sseq);
+  if ((cov->Sseq = (sequential_storage*) MALLOC(sizeof(sequential_storage))) 
+       == NULL ||
+      (U22 = (double *) MALLOC(sizeof(double) * totpntsSQ))==NULL ||
+      (Inv22 = (double *) MALLOC(sizeof(double) * totpntsSQ))==NULL ||
+      (COV21 = (double *) MALLOC(sizeof(double) * spatialpntsSQback))==NULL ||
+      (U11 = (double *) MALLOC(sizeof(double) * spatialpntsSQ))==NULL ||
+      (MuT = (double *) MALLOC(sizeof(double) * spatialpntsSQback))==NULL ||
+      (G = (double *) MALLOC(sizeof(double) * totpnts))==NULL ||
+      (res0 = (res_type *) MALLOC(sizeof(res_type) * 
+				  (totpnts + spatialpnts * initial))) ==NULL) {
+    err=ERRORMEMORYALLOCATION;  
     goto ErrorHandling;
-  if ((U11 =(double *) malloc(sizeof(double) * spatialpntsSQ))==NULL)
-    goto ErrorHandling;
-  if ((MuT =(double *) malloc(sizeof(double) * spatialpntsSQback))==NULL)
-    goto ErrorHandling;
-  // memory space for do_sequential:
-  if ((G = (double *) malloc(sizeof(double) * totpnts))==NULL)
-      goto ErrorHandling;
-  if ((res0 = (res_type *) malloc(sizeof(res_type) * 
-				(totpnts + spatialpnts * initial))) ==NULL)
-      goto ErrorHandling;
-  S->U22 = S->U11 = S->MuT = S->G = S->Cov21 = S->Inv22 = NULL;
+  }
+  S = cov->Sseq;
+  SEQU_NULL(S);
   S->res0 = NULL;
 
   if (loc->length[spatialdim] <= back) {
-    Err = ERRORTRIVIALTIMEDIRECTION;
-    goto ErrorHandling;
-  } else Err = NOERROR;
+    GERR("the grid in the last direction is too small; use method `direct' instead of `sequential'");
+  } else err = NOERROR;
 
   /* ************************* */
   /* create matrix of explicit */
   /*       x-coordinates       */
   /* ************************* */
 
-
+ 
   store = loc->length[spatialdim];
   loc->length[spatialdim] = back;
-  if (loc->grid) {
-     expandgrid(loc->xgr, loc->length, 
-		&xx, 
-		loc->timespacedim);
-     if (!loc->Time) meth->space = meth->sptime;
-  } else {
-    xtime2x(loc->x, loc->length[0], loc->T, loc->length[spatialdim],
-	    &xx, loc->timespacedim);
-    assert(loc->Time);
-  }
+  Transform2NoGrid(cov, &xx);
+  assert(loc->caniso == NULL);
   loc->length[spatialdim] = store;
  
-//  PrintModelInfo(cov); assert(false);
-
-
   /* ********************* */
   /* matrix creation part  */
   /* ********************* */
   long j, k, segment;
-  int  row, err, k0, k1, k2;
+  int  row, f77err, k0, k1, k2;
   double *y;
-  y = (double*) malloc(loc->timespacedim * sizeof(double));
+  y = (double*) MALLOC(loc->timespacedim * sizeof(double));
 
   // *** S22
 
-//  printf
+//  print
 
-  if (PL>4) PRINTF("covariance matrix...\n");
+  if (PL==PL_SUBDETAILS) { LPRINT("covariance matrix...\n"); }
   k = 0;
   for (k0 =i=0; i<totpnts; i++, k0+=dim) {
     k += i;
@@ -218,15 +216,25 @@ int init_sequential(method_type *meth){
       k1 = k0;
       for (d=0; d<dim; d++) {
 	y[d] = xx[k1++] - xx[k2++];
-//	printf("%f ", y[d]);
+//	print("%f ", y[d]);
       }
-      cf(y, cov, U22 + segment);
-//      printf("cov=%f\n", U22[segment]);
+      COV(y, next, U22 + segment);
+
+//      print("cov=%f\n", U22[segment]);
       U22[k++] = U22[segment];
       segment += totpnts;
     }
   }
 
+  /*
+  for (k=i=0; i<totpnts; i++) {
+    for (d=0; d<totpnts; d++) {
+      printf("%f ", U22[k++]); //
+    }
+    printf("\n"); //
+  }
+  */
+       
   /* Orndung von U22: 
       back ....1
    back
@@ -263,8 +271,7 @@ int init_sequential(method_type *meth){
 	for (d=0; d<spatialdim; d++) {
 	    y[d] = xx[k1++] - xx[k2++];
 	}
-	cf(y, cov,  COV21 + k);
-
+	COV(y, next, COV21 + k);	
 	k++; k2++;
     }
     k += withoutlast;
@@ -274,53 +281,26 @@ int init_sequential(method_type *meth){
   y = NULL;
 
 //  for (i=0; i<withoutlast * spatialpnts; i++) {
-//    printf("%3.4f ", COV21[i]);
+//    print("%3.4f ", COV21[i]);
 //  }
 //  assert(false);
 
 /*
-
-source("~/R/RF/RandomFields/tests/source.R")
-
-
-model <- list(
-              "+",
-              list("$", aniso=matrix(1:4, ncol=2),
-                   list("exp")
-                   ),
-              list("$", var=0.,
-                   list("nugget")
-                   )
-              )
-model <- list("$", aniso=matrix(c(1,0,0,1.1), ncol=2),
-                   list("exp")
-                   )
-#  library(RandomFields, lib"~/R/OLD");model <- list(list(model="exp", aniso=c(1,0,0,1.1), var=1))
-
-RFparameters(Print=5, exactness=FALSE)
-x <- c(0,1,0.4) # --> err 66
-x <- c(0,1,0.12) # -> U11 neg ## done !!
-xx <- seq(x[1],x[2],x[3])
-z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
-             every=10 ) 
-*/
-
-/*
-  PRINTF("U11\n");
+  LPRINT("U11\n");
   for (i=0; i<spatialpnts; i++) {
     for (j=0; j<spatialpnts; j++) {
-      PRINTF("%3.2f ", U11[i + j * spatialpnts]);
-   }
-    PRINTF("\n");
+      LPRINT("%3.2f ", U11[i + j * spatialpnts]);
+    }
+    LPRINT("\n");
   }
 
-  PRINTF("S22 %d %d\n", totpnts,spatialpnts );
+  LPRINT("S22 %d %d\n", totpnts,spatialpnts );
   for (i=0; i<totpnts; i++) {
-    PRINTF("%2d: ", i);
+    LPRINT("%2d: ", i);
     for (j=0; j<totpnts; j++) {
-      PRINTF("%3.2f ", U22[i + j * totpnts]);
+      LPRINT("%3.2f ", U22[i + j * totpnts]);
    }
-    PRINTF("\n");
+    LPRINT("\n");
   }
 */
 
@@ -330,59 +310,58 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
   /* ********************* */
 
   // *** sqrt S 22 
-  if (PL>4) PRINTF("Cholesky decomposition of Cov22...\n");
+  if (PL>=PL_STRUCTURE) { LPRINT("Cholesky decomposition of Cov22...\n"); }
   row=totpnts;
   // dchdc destroys the input matrix; upper half of U22 contains result!
   // dpotrf	F77_CALL(dpotrf)("Upper", &m, REAL(ans), &m, &i);
   //  assert(false);
-  F77_CALL(dpotrf)("Upper", &row, U22, &row, &err);
+  F77_CALL(dpotrf)("Upper", &row, U22, &row, &f77err);
 //  assert(false);
-  // F77_NAME(dchdc)(U22, &row, &row, G, NULL, &choljob, &err);
-  if (err!=NOERROR) {
-    if (PL>2)
-      PRINTF("Error code F77_CALL(dpotrf) = %d\n", err);
-    Err=ERRORDECOMPOSITION;
+  // F77_NAME(dchdc)(U22, &row, &row, G, NULL, &choljob, &f77err);
+  if (f77err!=NOERROR) {
+    if (PL>=PL_SUBIMPORTANT) 
+      {LPRINT("Error code F77_CALL(dpotrf) = %d\n", f77err);}
+    err=ERRORDECOMPOSITION;
     goto ErrorHandling;
   } 
 
   // for (i=0; i<withoutlast * spatialpnts; i++) {
-//    PRINTF("%3.4f ", COV21[i]);
+//    LPRINT("%3.4f ", COV21[i]);
 //  }
 //  assert(false);
   
   // *** inverse of S 22 
-  if (PL>4) PRINTF("inverse of Cov22...\n");
-  memcpy(Inv22, U22, sizeof(double) * totpntsSQ);
-  F77_CALL(dpotri)("Upper", &row, Inv22, &row, &err);
-  if (err!=NOERROR) {
-    if (PL>2)
-      PRINTF("Error code F77_CALL(dpotri) = %d\n", err);
-    Err=ERRORDECOMPOSITION;
+  if (PL>=PL_STRUCTURE) { LPRINT("inverse of Cov22...\n"); }
+  MEMCOPY(Inv22, U22, sizeof(double) * totpntsSQ);
+  F77_CALL(dpotri)("Upper", &row, Inv22, &row, &f77err);
+  if (f77err!=NOERROR) {
+    if (PL>=PL_SUBIMPORTANT) { LPRINT("Error code F77_CALL(dpotri) = %d\n", f77err); }
+    err=ERRORDECOMPOSITION;
     goto ErrorHandling;
   }
   for (k=i=0; i<totpnts; i++) {
     k += i;
     for (segment=i* totpnts+i; segment<totpntsSQ; segment += totpnts) {
 	Inv22[k++] = Inv22[segment]; 
-//	PRINTF("sg %d %f %d \n", k, Inv22[segment] , segment);
+//	LPRINT("sg %d %f %d \n", k, Inv22[segment] , segment);
     }
   }
 
 
 /*
-  PRINTF("C21\n");
+  LPRINT("C21\n");
   for (i=0; i<totpnts; i++) {
     for (j=0; j<spatialpnts; j++) {
-      PRINTF("%3.2f ", COV21[i + j * totpnts]);
+      LPRINT("%3.2f ", COV21[i + j * totpnts]);
    }
-    PRINTF("\n");
+    LPRINT("\n");
   }
 */  
 
 
   // *** MuT
 
-  if (PL>4) PRINTF("calculating matrix for the mean...\n");
+  if (PL>=PL_STRUCTURE) { LPRINT("calculating matrix for the mean...\n"); }
   l = 0;
   for (i=0; i<spatialpntsSQback; i+=totpnts) {
     for (j=0; j<totpntsSQ; j+=totpnts) {
@@ -394,18 +373,18 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
   }
 
 /*
-  PRINTF("MuT\n");
+  LPRINT("MuT\n");
   for (i=0; i<totpnts; i++) {
     for (j=0; j<spatialpnts; j++) {
-      PRINTF("%3.2f ", MuT[i + j * totpnts]);
+      LPRINT("%3.2f ", MuT[i + j * totpnts]);
    }
-    PRINTF("\n");
+    LPRINT("\n");
   }
 */
   //  assert(false);
  
   // *** C11
-  if (PL>4) PRINTF("calculating cov matrix...\n");
+  if (PL>=PL_STRUCTURE) { LPRINT("calculating cov matrix...\n"); }
   l = 0;
   for (i=0; i<spatialpntsSQback; i+=totpnts) {
     for (j=0; j<spatialpntsSQback; j+=totpnts) {
@@ -417,39 +396,40 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
   }
 
 /*
-  PRINTF("U11\n");
+  LPRINT("U11\n");
   for (i=0; i<spatialpnts; i++) {
     for (j=0; j<spatialpnts; j++) {
-      PRINTF("%3.2f ", U11[i + j * spatialpnts]);
+      LPRINT("%3.2f ", U11[i + j * spatialpnts]);
    }
-   PRINTF("\n");
+   LPRINT("\n");
   }
 */
 
 
   // *** sqrt C11
-  if (PL>4) PRINTF("Cholesky decomposition of Cov11...\n");
+  if (PL>=PL_STRUCTURE) { LPRINT("Cholesky decomposition of Cov11...\n"); }
   row=spatialpnts;
   // dchdc destroys the input matrix; upper half of U22 contains result!
   // dpotrf	F77_CALL(dpotrf)("Upper", &m, REAL(ans), &m, &i);
-  F77_CALL(dpotrf)("Upper", &row, U11, &row, &err);
+  F77_CALL(dpotrf)("Upper", &row, U11, &row, &f77err);
 
 /*
   for (i=0; i<spatialpnts; i++) {
     for (j=0; j<spatialpnts; j++) {
-      PRINTF("%3.2f ", U11[i + j * spatialpnts]);
+      LPRINT("%3.2f ", U11[i + j * spatialpnts]);
    }
-    PRINTF("\n");
+    LPRINT("\n");
   }
 */
 
-  if (err!=NOERROR) {
-    if (PL>2)
-      PRINTF("U11: Error code F77_CALL(dpotrf) = %d\n", err);
-    Err=ERRORDECOMPOSITION;
+  if (f77err!=NOERROR) {
+    if (PL>=PL_ERRORS) { LPRINT("U11: Error code F77_CALL(dpotrf) = %d\n", f77err); }
+    err=ERRORDECOMPOSITION;
     goto ErrorHandling;
   }
 
+  err = FieldReturn(cov);
+
  ErrorHandling: // and NOERROR...
   if (xx != NULL) free(xx);
   if (S!=NULL) {
@@ -459,7 +439,7 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
       S->initial = initial;
       S->ntime = loc->length[spatialdim];
   }
-  if (!storing && Err!=NOERROR) {
+  if (!storing && err!=NOERROR) {
     if (MuT!=NULL) free(MuT);
     if (U11!=NULL) free(U11);
     if (U22!=NULL) free(U22);
@@ -472,6 +452,7 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
       S->MuT=MuT;
       S->G=G;
       S->res0=res0;
+      //assert(false);
      }
   }
   if (COV21!=NULL) {
@@ -483,7 +464,11 @@ z <- GaussRF(x, x, model=model, gridtri=TRUE, grid=TRUE, Print=6, me="se",
       else free(Inv22);
   }
 
-  return Err;
+  cov->simu.active = err == NOERROR;
+
+  //printf("active=%d\n", cov->simu.active); assert(false);
+
+  return err;
 }
 
 
@@ -491,8 +476,8 @@ void sequentialpart(res_type *res, long totpnts, int spatialpnts, int ntime,
 		    double *U11, double *MuT, double *G) {
   res_type *rp, *oldrp;
   int n, i, k, j, mutj;
-  rp = &(res[totpnts]);
-  oldrp = &(res[0]);
+  rp = res + totpnts;
+  oldrp = res;
   for (n=0; n<ntime; n++, rp += spatialpnts, oldrp += spatialpnts) {
     for (i=0; i<spatialpnts; i++) G[i] = GAUSS_RANDOM(1.0);
     for (mutj=k=0, i=0; i<spatialpnts; i++, k+=spatialpnts) {
@@ -512,15 +497,26 @@ void sequentialpart(res_type *res, long totpnts, int spatialpnts, int ntime,
 }
 
 
-void do_sequential(method_type *meth, res_type *res) 
+void do_sequential(cov_model *cov, storage VARIABLE_IS_NOT_USED *s) 
 {  
-  sequential_storage *S;
-  long totpnts, i, j, k;
+  location_type *loc = Loc(cov);
+  sequential_storage
+    *S = cov->Sseq;
+  assert(S != NULL); 
+  long  i, j, k,
+    totpnts = S->totpnts;
   double *G,*U22, *U11, *MuT;
-  res_type *res0; 
+  res_type *res0,
+    *res = cov->rf; 
+  bool loggauss = (bool) ((int*) cov->p[LOG_GAUSS])[0];
 
-  S = (sequential_storage*) meth->S;
-  totpnts = S->totpnts;
+  assert(res != NULL);
+
+  assert(S != NULL); 
+
+ 
+  //  printf("totpnts %ld %d %d\n", totpnts, S->initial, S->spatialpnts);
+  // assert(false);
  
   U22 = S->U22;  // S22^{1/2}
   U11 = S->U11;  // S11^{1/2}
@@ -543,7 +539,14 @@ void do_sequential(method_type *meth, res_type *res)
   
   sequentialpart(res0, totpnts, S->spatialpnts, S->initial, U11, MuT, G);
   res0 += S->initial * S->spatialpnts;
-  memcpy(res, res0, sizeof(res_type) * totpnts);
+  MEMCOPY(res, res0, sizeof(res_type) * totpnts);
   sequentialpart(res, totpnts, S->spatialpnts, S->ntime - S->back, 
 		 U11, MuT, G);
+
+  if (loggauss) {
+    int vdimtot = loc->totalpoints * cov->vdim;
+    for (i=0; i<vdimtot; i++) res[i] = exp(res[i]);
+  }
+
+  
 }
